import 'slickgrid/slick.compositeeditor.js';
import { BindingEventService, Constants, getDescendantProperty, sanitizeTextByAvailableSanitizer, } from '@slickgrid-universal/common';
const DEFAULT_ON_ERROR = (error) => console.log(error.message);
export class SlickCompositeEditorComponent {
    constructor() {
        this._eventHandler = new Slick.EventHandler();
        this._bindEventService = new BindingEventService();
    }
    get eventHandler() {
        return this._eventHandler;
    }
    get dataView() {
        return this.grid.getData();
    }
    get dataViewLength() {
        return this.dataView.getLength();
    }
    get formValues() {
        return this._formValues;
    }
    get editors() {
        return this._editors;
    }
    set editors(editors) {
        this._editors = editors;
    }
    get gridOptions() {
        var _a;
        return (_a = this.grid) === null || _a === void 0 ? void 0 : _a.getOptions();
    }
    init(grid, containerService) {
        var _a, _b;
        this.grid = grid;
        this.gridService = containerService.get('GridService');
        this.gridStateService = containerService.get('GridStateService');
        this.translaterService = containerService.get('TranslaterService');
        if (!this.gridService || !this.gridStateService) {
            throw new Error('[Slickgrid-Universal] it seems that the GridService and/or GridStateService are not being loaded properly, make sure the Container Service is properly implemented.');
        }
        if (this.gridOptions.enableTranslate && (!this.translaterService || !this.translaterService.translate)) {
            throw new Error('[Slickgrid-Universal] requires a Translate Service to be installed and configured when the grid option "enableTranslate" is enabled.');
        }
        // get locales provided by user in forRoot or else use default English locales via the Constants
        this._locales = (_b = (_a = this.gridOptions) === null || _a === void 0 ? void 0 : _a.locales) !== null && _b !== void 0 ? _b : Constants.locales;
    }
    /** Dispose of the Component & unsubscribe all events */
    dispose() {
        this._eventHandler.unsubscribeAll();
        this._bindEventService.unbindAll();
        this._formValues = null;
        this.disposeComponent();
    }
    /** Dispose of the Component without unsubscribing any events */
    disposeComponent() {
        var _a;
        if (typeof ((_a = this._modalElm) === null || _a === void 0 ? void 0 : _a.remove) === 'function') {
            this._modalElm.remove();
            // remove the body backdrop click listener, every other listeners will be dropped automatically since we destroy the component
            document.body.classList.remove('slick-modal-open');
        }
    }
    /**
     * Dynamically change value of an input from the Composite Editor form
     * @param {String} columnId - column id
     * @param {*} newValue - the new value
     */
    changeFormInputValue(columnId, newValue) {
        var _a, _b, _c;
        const editor = (_a = this._editors) === null || _a === void 0 ? void 0 : _a[columnId];
        let outputValue = newValue;
        if (!editor) {
            throw new Error(`Editor with column id "${columnId}" not found.`);
        }
        if (editor && editor.setValue && Array.isArray(this._editorContainers)) {
            editor.setValue(newValue, true);
            const editorContainerElm = this._editorContainers.find((editorElm) => editorElm.dataset.editorid === columnId);
            if (!editor.disabled) {
                (_b = editorContainerElm === null || editorContainerElm === void 0 ? void 0 : editorContainerElm.classList) === null || _b === void 0 ? void 0 : _b.add('modified');
            }
            else {
                outputValue = '';
                (_c = editorContainerElm === null || editorContainerElm === void 0 ? void 0 : editorContainerElm.classList) === null || _c === void 0 ? void 0 : _c.remove('modified');
            }
            this._formValues = Object.assign(Object.assign({}, this._formValues), { [columnId]: outputValue });
        }
    }
    /**
     * Dynamically change an Editor option of the Composite Editor form
     * For example, a use case could be to dynamically change the "minDate" of another Date Editor in the Composite Editor form.
     * @param {String} columnId - column id
     * @param {*} newValue - the new value
     */
    changeFormEditorOption(columnId, optionName, newOptionValue) {
        var _a;
        const editor = (_a = this._editors) === null || _a === void 0 ? void 0 : _a[columnId];
        // change an Editor option (not all Editors have that method, so make sure it exists before trying to call it)
        if (editor === null || editor === void 0 ? void 0 : editor.changeEditorOption) {
            editor.changeEditorOption(optionName, newOptionValue);
        }
        else {
            throw new Error(`Editor with column id "${columnId}" not found OR the Editor does not support "changeEditorOption" (current only available with AutoComplete, Date, MultipleSelect & SingleSelect Editors).`);
        }
    }
    /**
     * Disable (or enable) an input of the Composite Editor form
     * @param columnId
     * @param isDisabled
     */
    disableFormInput(columnId, isDisabled = true) {
        var _a;
        const editor = (_a = this._editors) === null || _a === void 0 ? void 0 : _a[columnId];
        if ((editor === null || editor === void 0 ? void 0 : editor.disable) && Array.isArray(this._editorContainers)) {
            editor.disable(isDisabled);
        }
    }
    openDetails(options) {
        var _a, _b, _c, _d, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
        const onError = (_a = options.onError) !== null && _a !== void 0 ? _a : DEFAULT_ON_ERROR;
        const defaultOptions = {
            backdrop: 'static',
            showCloseButtonOutside: true,
            viewColumnLayout: 'auto',
            modalType: 'edit',
        };
        try {
            if (!this.grid || (this.grid.getEditorLock().isActive() && !this.grid.getEditorLock().commitCurrentEdit())) {
                return null;
            }
            this._options = Object.assign(Object.assign(Object.assign(Object.assign({}, defaultOptions), this.gridOptions.compositeEditorOptions), options), { labels: Object.assign(Object.assign({}, (_b = this.gridOptions.compositeEditorOptions) === null || _b === void 0 ? void 0 : _b.labels), options === null || options === void 0 ? void 0 : options.labels) }); // merge default options with user options
            this._options.backdrop = options.backdrop !== undefined ? options.backdrop : 'static';
            const viewColumnLayout = this._options.viewColumnLayout || 1;
            const activeCell = this.grid.getActiveCell();
            const activeColIndex = (_c = activeCell === null || activeCell === void 0 ? void 0 : activeCell.cell) !== null && _c !== void 0 ? _c : 0;
            const activeRow = (_d = activeCell === null || activeCell === void 0 ? void 0 : activeCell.row) !== null && _d !== void 0 ? _d : 0;
            const gridUid = this.grid.getUID() || '';
            let headerTitle = options.headerTitle || '';
            if (this.hasRowSelectionEnabled() && this._options.modalType === 'auto-mass' && this.grid.getSelectedRows) {
                const selectedRowsIndexes = this.grid.getSelectedRows() || [];
                if (selectedRowsIndexes.length > 0) {
                    this._options.modalType = 'mass-selection';
                    if (options === null || options === void 0 ? void 0 : options.headerTitleMassSelection) {
                        headerTitle = options === null || options === void 0 ? void 0 : options.headerTitleMassSelection;
                    }
                }
                else {
                    this._options.modalType = 'mass-update';
                    if (options === null || options === void 0 ? void 0 : options.headerTitleMassUpdate) {
                        headerTitle = options === null || options === void 0 ? void 0 : options.headerTitleMassUpdate;
                    }
                }
            }
            const modalType = this._options.modalType || 'edit';
            if (!this.gridOptions.editable) {
                onError({ type: 'error', code: 'EDITABLE_GRID_REQUIRED', message: 'Your grid must be editable in order to use the Composite Editor Modal.' });
                return null;
            }
            else if (!this.gridOptions.enableCellNavigation) {
                onError({ type: 'error', code: 'ENABLE_CELL_NAVIGATION_REQUIRED', message: 'Composite Editor requires the flag "enableCellNavigation" to be set to True in your Grid Options.' });
                return null;
            }
            else if (!this.gridOptions.enableAddRow && modalType === 'create') {
                onError({ type: 'error', code: 'ENABLE_ADD_ROW_REQUIRED', message: 'Composite Editor requires the flag "enableAddRow" to be set to True in your Grid Options when creating a new item.' });
                return null;
            }
            else if (!activeCell && modalType === 'edit') {
                onError({ type: 'warning', code: 'NO_RECORD_FOUND', message: 'No records selected for edit operation.' });
                return null;
            }
            else {
                const isWithMassChange = (modalType === 'mass-update' || modalType === 'mass-selection');
                const dataContext = !isWithMassChange ? this.grid.getDataItem(activeRow) : {};
                const columnDefinitions = this.grid.getColumns();
                const selectedRowsIndexes = this.hasRowSelectionEnabled() ? this.grid.getSelectedRows() : [];
                const fullDataset = (_g = (_f = this.dataView) === null || _f === void 0 ? void 0 : _f.getItems()) !== null && _g !== void 0 ? _g : [];
                const fullDatasetLength = (Array.isArray(fullDataset)) ? fullDataset.length : 0;
                this._lastActiveRowNumber = activeRow;
                const gridStateSelection = (_h = this.gridStateService) === null || _h === void 0 ? void 0 : _h.getCurrentRowSelections();
                const dataContextIds = (gridStateSelection === null || gridStateSelection === void 0 ? void 0 : gridStateSelection.dataContextIds) || [];
                // focus on a first cell with an Editor (unless current cell already has an Editor then do nothing)
                // also when it's a "Create" modal, we'll scroll to the end of the grid
                const rowIndex = modalType === 'create' ? this.dataViewLength : activeRow;
                const hasFoundEditor = this.focusOnFirstColumnCellWithEditor(columnDefinitions, dataContext, activeColIndex, rowIndex, isWithMassChange);
                if (!hasFoundEditor) {
                    return null;
                }
                if (modalType === 'edit' && !dataContext) {
                    onError({ type: 'warning', code: 'ROW_NOT_EDITABLE', message: 'Current row is not editable.' });
                    return null;
                }
                else if (modalType === 'mass-selection') {
                    if (selectedRowsIndexes.length < 1) {
                        onError({ type: 'warning', code: 'ROW_SELECTION_REQUIRED', message: 'You must select some rows before trying to apply new value(s).' });
                        return null;
                    }
                }
                let modalColumns = [];
                if (isWithMassChange) {
                    // when using Mass Update, we only care about the columns that have the "massUpdate: true", we disregard anything else
                    modalColumns = columnDefinitions.filter(col => { var _a; return col.editor && ((_a = col.internalColumnEditor) === null || _a === void 0 ? void 0 : _a.massUpdate) === true; });
                }
                else {
                    modalColumns = columnDefinitions.filter(col => col.editor);
                }
                // open the editor modal and we can also provide a header title with optional parsing pulled from the dataContext, via template {{ }}
                // for example {{title}} => display the item title, or even complex object works {{product.name}} => display item product name
                const parsedHeaderTitle = headerTitle.replace(/\{\{(.*?)\}\}/g, (_match, group) => getDescendantProperty(dataContext, group));
                const sanitizedHeaderTitle = sanitizeTextByAvailableSanitizer(this.gridOptions, parsedHeaderTitle);
                const layoutColCount = viewColumnLayout === 'auto' ? this.autoCalculateLayoutColumnCount(modalColumns.length) : viewColumnLayout;
                this._modalElm = document.createElement('div');
                this._modalElm.className = `slick-editor-modal ${gridUid}`;
                const modalContentElm = document.createElement('div');
                modalContentElm.className = 'slick-editor-modal-content';
                if (viewColumnLayout > 1 || (viewColumnLayout === 'auto' && layoutColCount > 1)) {
                    const splitClassName = layoutColCount === 2 ? 'split-view' : 'triple-split-view';
                    modalContentElm.classList.add(splitClassName);
                }
                const modalHeaderTitleElm = document.createElement('div');
                modalHeaderTitleElm.className = 'slick-editor-modal-title';
                modalHeaderTitleElm.innerHTML = sanitizedHeaderTitle;
                const modalCloseButtonElm = document.createElement('button');
                modalCloseButtonElm.type = 'button';
                modalCloseButtonElm.textContent = '×';
                modalCloseButtonElm.className = 'close';
                modalCloseButtonElm.dataset.action = 'close';
                modalCloseButtonElm.dataset.ariaLabel = 'Close';
                if (this._options.showCloseButtonOutside) {
                    (_j = modalHeaderTitleElm === null || modalHeaderTitleElm === void 0 ? void 0 : modalHeaderTitleElm.classList) === null || _j === void 0 ? void 0 : _j.add('outside');
                    (_k = modalCloseButtonElm === null || modalCloseButtonElm === void 0 ? void 0 : modalCloseButtonElm.classList) === null || _k === void 0 ? void 0 : _k.add('outside');
                }
                const modalHeaderElm = document.createElement('div');
                modalHeaderElm.className = 'slick-editor-modal-header';
                modalHeaderElm.appendChild(modalHeaderTitleElm);
                modalHeaderElm.appendChild(modalCloseButtonElm);
                const modalBodyElm = document.createElement('div');
                modalBodyElm.className = 'slick-editor-modal-body';
                this._modalBodyTopValidationElm = document.createElement('div');
                this._modalBodyTopValidationElm.className = 'validation-summary';
                this._modalBodyTopValidationElm.style.display = 'none';
                modalBodyElm.appendChild(this._modalBodyTopValidationElm);
                const modalFooterElm = document.createElement('div');
                modalFooterElm.className = 'slick-editor-modal-footer';
                const modalCancelButtonElm = document.createElement('button');
                modalCancelButtonElm.type = 'button';
                modalCancelButtonElm.className = 'btn btn-cancel btn-default btn-sm';
                modalCancelButtonElm.dataset.action = 'cancel';
                modalCancelButtonElm.dataset.ariaLabel = this.getLabelText('cancelButton', 'TEXT_CANCEL', 'Cancel');
                modalCancelButtonElm.textContent = this.getLabelText('cancelButton', 'TEXT_CANCEL', 'Cancel');
                let leftFooterText = '';
                let saveButtonText = '';
                switch (modalType) {
                    case 'mass-update':
                        const footerUnparsedText = this.getLabelText('massUpdateStatus', 'TEXT_ALL_X_RECORDS_SELECTED', 'All {{x}} records selected');
                        leftFooterText = this.parseText(footerUnparsedText, { x: fullDatasetLength });
                        saveButtonText = this.getLabelText('massUpdateButton', 'TEXT_APPLY_MASS_UPDATE', 'Mass Update');
                        break;
                    case 'mass-selection':
                        const selectionUnparsedText = this.getLabelText('massSelectionStatus', 'TEXT_X_OF_Y_MASS_SELECTED', '{{x}} of {{y}} selected');
                        leftFooterText = this.parseText(selectionUnparsedText, { x: dataContextIds.length, y: fullDatasetLength });
                        saveButtonText = this.getLabelText('massSelectionButton', 'TEXT_APPLY_TO_SELECTION', 'Update Selection');
                        break;
                    default:
                        saveButtonText = this.getLabelText('saveButton', 'TEXT_SAVE', 'Save');
                }
                const selectionCounterElm = document.createElement('div');
                selectionCounterElm.className = 'footer-status-text';
                selectionCounterElm.textContent = leftFooterText;
                this._modalSaveButtonElm = document.createElement('button');
                this._modalSaveButtonElm.type = 'button';
                this._modalSaveButtonElm.className = 'btn btn-save btn-primary btn-sm';
                this._modalSaveButtonElm.dataset.action = (modalType === 'create' || modalType === 'edit') ? 'save' : modalType;
                this._modalSaveButtonElm.dataset.ariaLabel = saveButtonText;
                this._modalSaveButtonElm.textContent = saveButtonText;
                const footerContainerElm = document.createElement('div');
                footerContainerElm.className = 'footer-buttons';
                if (modalType === 'mass-update' || modalType === 'mass-selection') {
                    modalFooterElm.appendChild(selectionCounterElm);
                }
                footerContainerElm.appendChild(modalCancelButtonElm);
                footerContainerElm.appendChild(this._modalSaveButtonElm);
                modalFooterElm.appendChild(footerContainerElm);
                modalContentElm.appendChild(modalHeaderElm);
                modalContentElm.appendChild(modalBodyElm);
                modalContentElm.appendChild(modalFooterElm);
                this._modalElm.appendChild(modalContentElm);
                for (const columnDef of modalColumns) {
                    if (columnDef.editor) {
                        const itemContainer = document.createElement('div');
                        itemContainer.className = `item-details-container editor-${columnDef.id}`;
                        if (layoutColCount === 1) {
                            itemContainer.classList.add('slick-col-medium-12');
                        }
                        else {
                            itemContainer.classList.add('slick-col-medium-6', `slick-col-xlarge-${12 / layoutColCount}`);
                        }
                        const templateItemLabelElm = document.createElement('div');
                        templateItemLabelElm.className = `item-details-label editor-${columnDef.id}`;
                        templateItemLabelElm.textContent = this.getColumnLabel(columnDef) || 'n/a';
                        const templateItemEditorElm = document.createElement('div');
                        templateItemEditorElm.className = 'item-details-editor-container slick-cell';
                        templateItemEditorElm.dataset.editorid = `${columnDef.id}`;
                        const templateItemValidationElm = document.createElement('div');
                        templateItemValidationElm.className = `item-details-validation editor-${columnDef.id}`;
                        itemContainer.appendChild(templateItemLabelElm);
                        itemContainer.appendChild(templateItemEditorElm);
                        itemContainer.appendChild(templateItemValidationElm);
                        modalBodyElm.appendChild(itemContainer);
                    }
                }
                document.body.appendChild(this._modalElm);
                document.body.classList.add('slick-modal-open'); // add backdrop to body
                this._bindEventService.bind(document.body, 'click', this.handleBodyClicked.bind(this));
                this._editors = {};
                this._editorContainers = modalColumns.map(col => modalBodyElm.querySelector(`[data-editorid=${col.id}]`)) || [];
                const compositeOptions = { destroy: this.disposeComponent.bind(this), modalType, validationMsgPrefix: '* ', formValues: {}, editors: this._editors };
                const compositeEditor = new Slick.CompositeEditor(modalColumns, this._editorContainers, compositeOptions);
                this.grid.editActiveCell(compositeEditor);
                // --
                // Add a few Event Handlers
                // keyboard, blur & button event handlers
                this._bindEventService.bind(modalCloseButtonElm, 'click', this.cancelEditing.bind(this));
                this._bindEventService.bind(modalCancelButtonElm, 'click', this.cancelEditing.bind(this));
                this._bindEventService.bind(this._modalSaveButtonElm, 'click', this.handleSaveClicked.bind(this));
                this._bindEventService.bind(this._modalElm, 'keydown', this.handleKeyDown.bind(this));
                this._bindEventService.bind(this._modalElm, 'focusout', this.validateCurrentEditor.bind(this));
                this._bindEventService.bind(this._modalElm, 'blur', this.validateCurrentEditor.bind(this));
                // when any of the input of the composite editor form changes, we'll add/remove a "modified" CSS className for styling purposes
                const onCompositeEditorChangeHandler = this.grid.onCompositeEditorChange;
                this._eventHandler
                    .subscribe(onCompositeEditorChangeHandler, this.handleOnCompositeEditorChange.bind(this));
                // when adding a new row to the grid, we need to invalidate that row and re-render the grid
                const onAddNewRowHandler = this.grid.onAddNewRow;
                this._eventHandler
                    .subscribe(onAddNewRowHandler, this.handleOnAddNewRow.bind(this));
            }
            return this;
        }
        catch (error) {
            this.dispose();
            const errorMsg = (typeof error === 'string') ? error : ((_o = (_l = error === null || error === void 0 ? void 0 : error.message) !== null && _l !== void 0 ? _l : (_m = error === null || error === void 0 ? void 0 : error.body) === null || _m === void 0 ? void 0 : _m.message) !== null && _o !== void 0 ? _o : '');
            const errorCode = (typeof error === 'string') ? error : (_r = (_p = error === null || error === void 0 ? void 0 : error.status) !== null && _p !== void 0 ? _p : (_q = error === null || error === void 0 ? void 0 : error.body) === null || _q === void 0 ? void 0 : _q.status) !== null && _r !== void 0 ? _r : errorMsg;
            onError({ type: 'error', code: errorCode, message: errorMsg });
            return null;
        }
    }
    /** Apply Mass Update Changes from a the form changes */
    applySaveMassUpdateChanges(formValues) {
        const data = this.dataView.getItems();
        // from the "lastCompositeEditor" object that we kept as reference, it contains all the changes inside the "formValues" property
        // we can loop through these changes and apply them on the selected row indexes
        for (const itemProp in formValues) {
            if (formValues.hasOwnProperty(itemProp)) {
                data.forEach(item => {
                    if (formValues.hasOwnProperty(itemProp)) {
                        item[itemProp] = formValues[itemProp];
                    }
                });
            }
        }
        // change the entire dataset with our updated dataset
        this.dataView.setItems(data, this.gridOptions.datasetIdPropertyName);
        this.grid.invalidate();
    }
    applySaveMassSelectionChanges(formValues, selection) {
        var _a, _b;
        const selectedItemIds = (_a = selection === null || selection === void 0 ? void 0 : selection.dataContextIds) !== null && _a !== void 0 ? _a : [];
        const selectedItems = selectedItemIds.map(itemId => this.dataView.getItemById(itemId));
        // from the "lastCompositeEditor" object that we kept as reference, it contains all the changes inside the "formValues" property
        // we can loop through these changes and apply them on the selected row indexes
        for (const itemProp in formValues) {
            if (formValues.hasOwnProperty(itemProp)) {
                selectedItems.forEach(dataContext => {
                    if (dataContext && dataContext.hasOwnProperty(itemProp) && formValues.hasOwnProperty(itemProp)) {
                        dataContext[itemProp] = formValues[itemProp];
                    }
                });
            }
        }
        // update all items in the grid with the grid service
        (_b = this.gridService) === null || _b === void 0 ? void 0 : _b.updateItems(selectedItems);
    }
    async cancelEditing() {
        let confirmed = true;
        if (this.formValues && Object.keys(this.formValues).length > 0 && typeof this._options.onClose === 'function') {
            confirmed = await this._options.onClose();
        }
        if (confirmed) {
            this.grid.getEditController().cancelCurrentEdit();
            this.grid.setActiveRow(this._lastActiveRowNumber);
            this.dispose();
        }
    }
    handleSaveClicked() {
        var _a;
        const modalType = (_a = this._options) === null || _a === void 0 ? void 0 : _a.modalType;
        switch (modalType) {
            case 'mass-update':
                this.handleMassSaving(modalType, () => {
                    this.grid.getEditController().cancelCurrentEdit();
                    this.grid.setActiveCell(0, 0, false);
                    this.dispose();
                });
                break;
            case 'mass-selection':
                this.handleMassSaving(modalType, () => {
                    this.grid.getEditController().cancelCurrentEdit();
                    this.grid.setActiveRow(this._lastActiveRowNumber);
                    this.dispose();
                });
                break;
            case 'create':
            case 'edit':
            default:
                this.grid.getEditController().commitCurrentEdit();
                break;
        }
    }
    async handleMassSaving(modalType, executeCallback) {
        var _a, _b, _c, _d, _f, _g;
        const validationResults = this.validateCompositeEditors();
        const isFormValid = validationResults.valid;
        this.showValidationSummaryText(false, '');
        const applyCallbackFnName = (modalType === 'mass-update') ? 'applySaveMassUpdateChanges' : 'applySaveMassSelectionChanges';
        try {
            if (typeof this[applyCallbackFnName] === 'function') {
                if (!this.formValues || Object.keys(this.formValues).length === 0) {
                    this.executeOnError({ type: 'warning', code: 'NO_CHANGES_DETECTED', message: 'Sorry we could not detect any changes.' });
                }
                else if (isFormValid && this.formValues) {
                    this._modalSaveButtonElm.classList.add('saving');
                    this._modalSaveButtonElm.disabled = true;
                    const gridStateSelection = (_a = this.gridStateService) === null || _a === void 0 ? void 0 : _a.getCurrentRowSelections();
                    const gridRowIndexes = (gridStateSelection === null || gridStateSelection === void 0 ? void 0 : gridStateSelection.gridRowIndexes) || [];
                    const dataContextIds = (gridStateSelection === null || gridStateSelection === void 0 ? void 0 : gridStateSelection.dataContextIds) || [];
                    if (typeof ((_b = this._options) === null || _b === void 0 ? void 0 : _b.onSave) === 'function') {
                        const successful = await ((_c = this._options) === null || _c === void 0 ? void 0 : _c.onSave(this.formValues, { gridRowIndexes, dataContextIds }, this[applyCallbackFnName].bind(this)));
                        if (successful) {
                            // once we're done doing the mass update, we can cancel the current editor since we don't want to add any new row
                            // that will also destroy/close the modal window
                            executeCallback();
                        }
                    }
                    else {
                        this[applyCallbackFnName](this.formValues, { gridRowIndexes, dataContextIds });
                        executeCallback();
                        this.dispose();
                    }
                    this._modalSaveButtonElm.disabled = false;
                    this._modalSaveButtonElm.classList.remove('saving');
                }
            }
        }
        catch (error) {
            const errorMsg = (typeof error === 'string') ? error : ((_g = (_d = error === null || error === void 0 ? void 0 : error.message) !== null && _d !== void 0 ? _d : (_f = error === null || error === void 0 ? void 0 : error.body) === null || _f === void 0 ? void 0 : _f.message) !== null && _g !== void 0 ? _g : '');
            this.showValidationSummaryText(true, errorMsg);
        }
    }
    showValidationSummaryText(isShowing, errorMsg = '') {
        var _a, _b;
        if (isShowing) {
            this._modalBodyTopValidationElm.textContent = errorMsg;
            this._modalBodyTopValidationElm.style.display = 'block';
            (_b = (_a = this._modalBodyTopValidationElm).scrollIntoView) === null || _b === void 0 ? void 0 : _b.call(_a);
            this._modalSaveButtonElm.disabled = false;
            this._modalSaveButtonElm.classList.remove('saving');
        }
        else {
            this._modalBodyTopValidationElm.style.display = 'none';
            this._modalBodyTopValidationElm.textContent = errorMsg;
        }
    }
    validateCurrentEditor() {
        const currentEditor = this.grid.getCellEditor();
        if (currentEditor === null || currentEditor === void 0 ? void 0 : currentEditor.validate) {
            currentEditor.validate();
        }
    }
    // --
    // private methods
    // ----------------
    /**
     * Auto-Calculate how many columns to display in the view layout (1, 2, or 3).
     * We'll display a 1 column layount for 8 or less Editors, 2 columns layout for less than 15 Editors or 3 columns when more than 15 Editors
     * @param {number} editorCount - how many Editors do we have in total
     * @returns {number} count - calculated column count (1, 2 or 3)
     */
    autoCalculateLayoutColumnCount(editorCount) {
        if (editorCount >= 15) {
            return 3;
        }
        else if (editorCount >= 8) {
            return 2;
        }
        return 1;
    }
    executeOnError(error) {
        var _a, _b;
        const onError = (_b = (_a = this._options) === null || _a === void 0 ? void 0 : _a.onError) !== null && _b !== void 0 ? _b : DEFAULT_ON_ERROR;
        onError(error);
    }
    /**
     * Get the column label, the label might have an optional "columnGroup" (or "columnGroupKey" which need to be translated)
     * @param {object} columnDef - column definition
     * @returns {string} label - column label
     */
    getColumnLabel(columnDef) {
        var _a;
        const columnGroupSeparator = this.gridOptions.columnGroupSeparator || ' - ';
        let columnName = columnDef.name || '';
        let columnGroup = columnDef.columnGroup || '';
        if (this.gridOptions.enableTranslate && this.translaterService) {
            if (columnDef.nameKey) {
                columnName = this.translaterService.translate(columnDef.nameKey);
            }
            if (columnDef.columnGroupKey && ((_a = this.translaterService) === null || _a === void 0 ? void 0 : _a.translate)) {
                columnGroup = this.translaterService.translate(columnDef.columnGroupKey);
            }
        }
        const columnLabel = columnGroup ? `${columnGroup}${columnGroupSeparator}${columnName}` : columnName;
        return columnLabel || '';
    }
    handleBodyClicked(event) {
        var _a, _b, _c;
        if ((_b = (_a = event.target) === null || _a === void 0 ? void 0 : _a.classList) === null || _b === void 0 ? void 0 : _b.contains('slick-editor-modal')) {
            if (((_c = this._options) === null || _c === void 0 ? void 0 : _c.backdrop) !== 'static') {
                this.dispose();
            }
        }
    }
    handleKeyDown(event) {
        if (event.code === 'Escape') {
            this.cancelEditing();
            event.stopPropagation();
            event.preventDefault();
        }
        else if (event.code === 'Tab') {
            this.validateCurrentEditor();
        }
    }
    // For the Composite Editor to work, the current active cell must have an Editor (because it calls editActiveCell() and that only works with a cell with an Editor)
    // so if current active cell doesn't have an Editor, we'll find the first column with an Editor and focus on it (from left to right starting at index 0)
    focusOnFirstColumnCellWithEditor(columns, dataContext, columnIndex, rowIndex, isWithMassChange) {
        let columnIndexWithEditor = columnIndex;
        const cellEditor = columns[columnIndex].editor;
        if (!cellEditor || !this.getActiveCellEditor(rowIndex, columnIndex)) {
            columnIndexWithEditor = this.findNextAvailableEditorColumnIndex(columns, dataContext, rowIndex, isWithMassChange);
            if (columnIndexWithEditor === -1) {
                this.executeOnError({ type: 'error', code: 'NO_EDITOR_FOUND', message: 'We could not find any Editor in your Column Definition' });
                return false;
            }
            else {
                this.grid.setActiveCell(rowIndex, columnIndexWithEditor, false);
                if (isWithMassChange) {
                    // when it's a mass change, we'll activate the last row without scrolling to it
                    // that is possible via the 3rd argument "suppressScrollIntoView" set to "true"
                    this.grid.setActiveRow(this.dataViewLength, columnIndexWithEditor, true);
                }
            }
        }
        return true;
    }
    findNextAvailableEditorColumnIndex(columns, dataContext, rowIndex, isWithMassUpdate) {
        var _a;
        let columnIndexWithEditor = -1;
        for (let colIndex = 0; colIndex < columns.length; colIndex++) {
            const col = columns[colIndex];
            if (col.editor && (!isWithMassUpdate || (isWithMassUpdate && ((_a = col.internalColumnEditor) === null || _a === void 0 ? void 0 : _a.massUpdate)))) {
                // we can check that the cell is really editable by checking the onBeforeEditCell event not returning false (returning undefined, null also mean it is editable)
                const isCellEditable = this.grid.onBeforeEditCell.notify({ row: rowIndex, cell: colIndex, item: dataContext, column: col, grid: this.grid });
                this.grid.setActiveCell(rowIndex, colIndex, false);
                if (isCellEditable !== false) {
                    columnIndexWithEditor = colIndex;
                    break;
                }
            }
        }
        return columnIndexWithEditor;
    }
    getActiveCellEditor(row, cell) {
        this.grid.setActiveCell(row, cell, false);
        return this.grid.getCellEditor();
    }
    getLabelText(labelProperty, localeText, defaultText) {
        var _a, _b, _c, _d;
        const textLabels = Object.assign(Object.assign({}, (_a = this.gridOptions.compositeEditorOptions) === null || _a === void 0 ? void 0 : _a.labels), (_b = this._options) === null || _b === void 0 ? void 0 : _b.labels);
        if (((_c = this.gridOptions) === null || _c === void 0 ? void 0 : _c.enableTranslate) && ((_d = this.translaterService) === null || _d === void 0 ? void 0 : _d.translate) && textLabels.hasOwnProperty(`${labelProperty}Key`)) {
            const translationKey = textLabels[`${labelProperty}Key`];
            return this.translaterService.translate(translationKey);
        }
        return (textLabels && textLabels[labelProperty]) || (this._locales && this._locales[localeText]) || defaultText;
    }
    handleOnAddNewRow(_e, args) {
        var _a, _b, _c, _d;
        const fullDataset = (_b = (_a = this.dataView) === null || _a === void 0 ? void 0 : _a.getItems()) !== null && _b !== void 0 ? _b : [];
        const fullDatasetLength = (Array.isArray(fullDataset)) ? fullDataset.length : 0;
        const newId = (_c = this._options.insertNewId) !== null && _c !== void 0 ? _c : fullDatasetLength + 1;
        const item = args.item;
        item[this.gridOptions.datasetIdPropertyName || 'id'] = newId;
        if (!this.dataView.getItemById(newId)) {
            (_d = this.gridService) === null || _d === void 0 ? void 0 : _d.addItem(item, this._options.insertOptions);
            this.dispose();
        }
    }
    /** Anytime an input of the Composite Editor form changes, we'll add/remove a "modified" CSS className for styling purposes */
    handleOnCompositeEditorChange(_e, args) {
        var _a, _b, _c, _d, _f;
        const columnId = (_b = (_a = args.column) === null || _a === void 0 ? void 0 : _a.id) !== null && _b !== void 0 ? _b : '';
        this._formValues = args.formValues;
        const editor = (_c = this._editors) === null || _c === void 0 ? void 0 : _c[columnId];
        const isEditorValueChanged = (_f = (_d = editor === null || editor === void 0 ? void 0 : editor.isValueChanged) === null || _d === void 0 ? void 0 : _d.call(editor)) !== null && _f !== void 0 ? _f : false;
        // add extra css styling to the composite editor input(s) that got modified
        const editorElm = document.querySelector(`[data-editorid=${columnId}]`);
        if (editorElm === null || editorElm === void 0 ? void 0 : editorElm.classList) {
            if (isEditorValueChanged) {
                editorElm.classList.add('modified');
            }
            else {
                editorElm.classList.remove('modified');
            }
        }
        // after any input changes we'll re-validate all fields
        this.validateCompositeEditors();
    }
    /** Check wether the grid has the Row Selection enabled */
    hasRowSelectionEnabled() {
        const selectionModel = this.grid.getSelectionModel();
        const isRowSelectionEnabled = this.gridOptions.enableRowSelection || this.gridOptions.enableCheckboxSelector;
        return (isRowSelectionEnabled && selectionModel);
    }
    parseText(inputText, mappedArgs) {
        return inputText.replace(/\{\{(.*?)\}\}/g, (match, group) => {
            return mappedArgs[group] !== undefined ? mappedArgs[group] : match;
        });
    }
    validateCompositeEditors(targetElm) {
        let validationResults = { valid: true, msg: '' };
        const currentEditor = this.grid.getCellEditor();
        if (currentEditor) {
            validationResults = currentEditor.validate(targetElm);
        }
        return validationResults;
    }
}
//# sourceMappingURL=slick-composite-editor.component.js.map